;/////////////////////////////////////////////////////////////;
;                                                             ;
;                 SimpleDX11 Model Procedures                 ;
;                                                             ;
;                                                             ;
;                   (c) 2013 SimpleDX Team                    ;
;                                                             ;
;                         License: MIT                        ;
;                                                             ;
;/////////////////////////////////////////////////////////////;



;-//    Private structures    //
Structure Model_Template
  *vTable
  *p.Model
  *SD3D.Screen3D
  *Primitive.Primitive
  *Mesh.Mesh
  *Texture.Texture
  
  *VertexArray
  *IntexArray  
  
  Stride.l
  Topology.l 
  VertexCount.l   

EndStructure


;-//    Constructor    //
Procedure New_Model(*SD3D, Flags.l = #Null)
  Protected *this.Model_Template
 
  *this = AllocateMemory(SizeOf(Model_Template))
  InitializeStructure(*this, Model_Template)
  
  If *this
    *this\vTable            = ?Model_Class    
    *this\p                 = *this
    *this\SD3D               = *SD3D    

    *this\Primitive         = New_Primitive(Flags)
    *this\Mesh              = New_Mesh(*this\SD3D, Flags) 
    
    If Not *this\Primitive Or Not *this\Mesh  
      ProcedureReturn #False 
    EndIf      

      
    ProcedureReturn *this
  EndIf  
  ProcedureReturn #False  
EndProcedure




; Get the models primitive object 
Procedure Model_GetPrimitive(*this.Model_Template)
  ProcedureReturn *this\Primitive 
EndProcedure

; Get the models mesh object 
Procedure Model_GetMesh(*this.Model_Template)
  ProcedureReturn *this\Mesh 
EndProcedure

Procedure Model_LoadTexture(*this.Model_Template, File.s)
  *this\Texture = New_Texture(*this\SD3D)
  If *this\Texture\CreateTextureFromFile(File)    
    ProcedureReturn #False  
  EndIf  
  
  ProcedureReturn *this\Texture\GetTexture()
EndProcedure

; Add a triangle to the model
Procedure Model_AddTriangle(*this.Model_Template, TriangleSize.f, ColorValue = #Null)
  Protected *Color.D3DXCOLOR = AllocateMemory(SizeOf(D3DXCOLOR))
  
  GetColorFloats() 

  *this\Primitive\AddVertex(-TriangleSize, -TriangleSize, 0.0)  
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 1.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0)   
  
  *this\Primitive\AddVertex(0.0, TriangleSize, 0.0)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.5, 0.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0) 
  
  *this\Primitive\AddVertex(TriangleSize, -TriangleSize, 0.0)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 1.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0) 
  
  FreeMemory(*Color)
  
  *this\VertexCount = 3
  
  Dim Indices.l(2)
  Indices(0) = 0
  Indices(1) = 1
  Indices(2) = 2    
  
  *this\Primitive\AddIndicesArray(Indices(), 2)    

  FreeArray(Indices())
EndProcedure

; Add a Rectangle to the model
Procedure Model_AddRectangle(*this.Model_Template, RectangleSize.f, ColorValue = #Null)
  Protected *Color.D3DXCOLOR = AllocateMemory(SizeOf(D3DXCOLOR))
  
  GetColorFloats()
  
  *this\Primitive\AddVertex(-RectangleSize, RectangleSize, 0.0)  
  *this\Primitive\SetColor(*Color)  
  *this\Primitive\SetUV(0.0, 0.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0) 
  
  *this\Primitive\AddVertex(RectangleSize, -RectangleSize, 0.0)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 1.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0) 
  
  *this\Primitive\AddVertex(-RectangleSize, -RectangleSize, 0.0)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 1.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0) 
  
  *this\Primitive\AddVertex(-RectangleSize, RectangleSize, 0.0)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 0.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0) 
  
  *this\Primitive\AddVertex(RectangleSize, RectangleSize, 0.0)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 0.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0)    
  
  *this\Primitive\AddVertex(RectangleSize, -RectangleSize, 0.0)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 1.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0)   
  
  FreeMemory(*Color)
  
  *this\VertexCount = 6
  
  Dim Indices.l(5)
  Indices(0) = 0
  Indices(1) = 1
  Indices(2) = 2    
  Indices(3) = 3
  Indices(4) = 4
  Indices(5) = 5   
  
  *this\Primitive\AddIndicesArray(Indices(), 5)    

  FreeArray(Indices())
EndProcedure


; Add a Cube to the model
Procedure Model_AddCube(*this.Model_Template, CubeSize.f,  ColorValue = #Null)
  Protected *Color.D3DXCOLOR = AllocateMemory(SizeOf(D3DXCOLOR)) 
  
  GetColorFloats() 


	;Front Face
	;///////////////////////////////////////////////	
	*this\Primitive\AddVertex(-CubeSize, -CubeSize, -CubeSize )  
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 1.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0) 	
	
  *this\Primitive\AddVertex(-CubeSize, CubeSize, -CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 0.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0)  

  *this\Primitive\AddVertex(CubeSize, CubeSize, -CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 0.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0)    

  *this\Primitive\AddVertex(CubeSize, -CubeSize, -CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 1.0)
  *this\Primitive\SetNormal(0.0, 0.0, -1.0)    

  ;Back Face
  ;///////////////////////////////////////////////	
  *this\Primitive\AddVertex(-CubeSize, -CubeSize, CubeSize)  
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 1.0)
  *this\Primitive\SetNormal(0.0, 0.0, 1.0)

  *this\Primitive\AddVertex(CubeSize, -CubeSize, CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 1.0)
  *this\Primitive\SetNormal(0.0, 0.0, 1.0)

  *this\Primitive\AddVertex(CubeSize, CubeSize, CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 0.0)
  *this\Primitive\SetNormal(0.0, 0.0, 1.0)

  *this\Primitive\AddVertex(-CubeSize, CubeSize, CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 0.0)
  *this\Primitive\SetNormal(0.0, 0.0, 1.0)

  ;Top Face
  ;///////////////////////////////////////////////	
  *this\Primitive\AddVertex(-CubeSize, CubeSize, -CubeSize)  
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 1.0)
  *this\Primitive\SetNormal(0.0, 1.0, 0.0)

  *this\Primitive\AddVertex(-CubeSize, CubeSize, CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 0.0)
  *this\Primitive\SetNormal(0.0, 1.0, 0.0)

  *this\Primitive\AddVertex(CubeSize, CubeSize, CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 0.0)
  *this\Primitive\SetNormal(0.0, 1.0, 0.0)

  *this\Primitive\AddVertex(CubeSize, CubeSize, -CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 1.0)
  *this\Primitive\SetNormal(0.0, 1.0, 0.0)

  ;Bottom Face
  ;///////////////////////////////////////////////	
  *this\Primitive\AddVertex(-CubeSize, -CubeSize, -CubeSize) 
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 1.0)
  *this\Primitive\SetNormal(0.0, -1.0, 0.0)

  *this\Primitive\AddVertex(CubeSize, -CubeSize, -CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 1.0)
  *this\Primitive\SetNormal(0.0, -1.0, 0.0)

  *this\Primitive\AddVertex(CubeSize, -CubeSize, CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 0.0)
  *this\Primitive\SetNormal(0.0, -1.0, 0.0)

  *this\Primitive\AddVertex(-CubeSize, -CubeSize, CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 0.0)
  *this\Primitive\SetNormal(0.0, -1.0, 0.0)

  ;Left Face
  ;///////////////////////////////////////////////	
  *this\Primitive\AddVertex(-CubeSize, -CubeSize, CubeSize)  
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 1.0)
  *this\Primitive\SetNormal(-1.0, 0.0, 0.0) 
  
  *this\Primitive\AddVertex(-CubeSize, CubeSize, CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 0.0)
  *this\Primitive\SetNormal(-1.0, 0.0, -1.0)

  *this\Primitive\AddVertex(-CubeSize, CubeSize, -CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 0.0)
  *this\Primitive\SetNormal(-1.0, 0.0, 0.0)

  *this\Primitive\AddVertex(-CubeSize, -CubeSize, -CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 1.0)
  *this\Primitive\SetNormal(-1.0, 0.0, 0.0)
		
  ;Right Face
  ;///////////////////////////////////////////////	
  *this\Primitive\AddVertex(CubeSize, -CubeSize, -CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 1.0)
  *this\Primitive\SetNormal(1.0, 0.0, 0.0)  

  *this\Primitive\AddVertex(CubeSize, CubeSize, -CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(0.0, 0.0)
  *this\Primitive\SetNormal(1.0, 0.0, 0.0) 

  *this\Primitive\AddVertex(CubeSize, CubeSize, CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 0.0)
  *this\Primitive\SetNormal(1.0, 0.0, 0.0)  

  *this\Primitive\AddVertex(CubeSize, -CubeSize, CubeSize)
  *this\Primitive\SetColor(*Color)
  *this\Primitive\SetUV(1.0, 1.0)
  *this\Primitive\SetNormal(1.0, 0.0, 0.0)  
 
  *this\VertexCount = 36
  
  Dim Indices.l(35)
  Indices(0) = 0
  Indices(1) = 1
  Indices(2) = 2    
  Indices(3) = 0
  Indices(4) = 2
  Indices(5) = 3
  
  Indices(6) = 4
  Indices(7) = 5
  Indices(8) = 6    
  Indices(9) = 4
  Indices(10) = 6
  Indices(11) = 7
  
  Indices(12) = 8
  Indices(13) = 9
  Indices(14) = 10    
  Indices(15) = 8
  Indices(16) = 10
  Indices(17) = 11 
  
  Indices(18) = 12
  Indices(19) = 13
  Indices(20) = 14    
  Indices(21) = 12
  Indices(22) = 14
  Indices(23) = 15  

  Indices(24) = 16
  Indices(25) = 17
  Indices(26) = 18    
  Indices(27) = 16
  Indices(28) = 18
  Indices(29) = 19 

  Indices(30) = 20
  Indices(31) = 21
  Indices(32) = 22    
  Indices(33) = 20
  Indices(34) = 22
  Indices(35) = 23   
  
  *this\Primitive\AddIndicesArray(Indices(), 35)    

  FreeArray(Indices())
	
EndProcedure

; Load a model from an *.obj file (wavefront.obj) 
Procedure Model_LoadObjModel(*this.Model_Template, File.s, ObjectName.s = "-88888888888")
  If FileSize(File) = -1 Or FileSize(File) = -2 : ProcedureReturn #False: EndIf
  
  Protected DataStr.s, Token.s
  Protected NewList tmpVertex.XMFLOAT3(), NewList tmpNormals.XMFLOAT3(), NewList tmpUVMap.UVMAP()
  Protected NewList tmpIndexList.s(), ObjectCount.l = 0, objName.s = "-9999999999999"
  Protected FilePath.s = GetPathPart(File), mtlFile.s
  Define *tmpColor.D3DXCOLOR = AllocateMemory(SizeOf(D3DXCOLOR))
  
  Protected FileID.l = ReadFile(#PB_Any, File)
  If FileID
    While Not Eof(FileID)       
      DataStr = ReadString(FileID)
      Token = LCase(StringField(DataStr, 1, " "))      
      
      Select Token
         Case "o"             
            If ObjectName = "" And ObjectCount > 0 : Break : EndIf
            If objName = ObjectName : Break : EndIf
            objName = StringField(DataStr, 2, " ")            
            ObjectCount   + 1 
            ClearList(tmpVertex())
            ClearList(tmpNormals())
            ClearList(tmpUVMap())
            ClearList(tmpIndexList())  
          Case "v"
            AddElement(tmpVertex())
            tmpVertex()\x = ValF(StringField(DataStr, 2, " "))
            tmpVertex()\y = ValF(StringField(DataStr, 3, " "))
            tmpVertex()\z = ValF(StringField(DataStr, 4, " "))
            
          Case "vn"
            AddElement(tmpNormals())
            tmpNormals()\x = ValF(StringField(DataStr, 2, " "))
            tmpNormals()\y = ValF(StringField(DataStr, 3, " "))
            tmpNormals()\z = ValF(StringField(DataStr, 4, " "))  
            
          Case "vt"  
            AddElement(tmpUVMap())
            tmpUVMap()\u = ValF(StringField(DataStr, 2, " "))
            tmpUVMap()\v = ValF(StringField(DataStr, 3, " "))            
            
          Case "mtllib"
            mtlFile = FilePath + StringField(DataStr, 2, " ")
            
          Case "usemtl"
            Protected Material.s = LCase(StringField(DataStr, 2, " "))            
            If FileSize(mtlFile) = -1 Or FileSize(mtlFile) = -2 : ProcedureReturn #False: EndIf            
            Protected mtlFileID.l = ReadFile(#PB_Any, mtlFile)            
            
            If mtlFileID
              Protected mtlStr.s, mtlToken.s, mtlMaterial.s
              While Not Eof(mtlFileID)
                
                mtlStr   = ReadString(mtlFileID)
                mtlToken = LCase(StringField(mtlStr, 1, " ")) 
                
                Select mtlToken
                  Case "newmtl"  
                    If Material = mtlMaterial : Break : EndIf
                    *tmpColor\r = #Null
                    *tmpColor\g = #Null
                    *tmpColor\b = #Null
                    *tmpColor\a = #Null                    
                    mtlMaterial = LCase(StringField(mtlStr, 2, " "))                     
                  Case "kd"                     
                    *tmpColor\r = ValF(StringField(mtlStr, 2, " "))
                    *tmpColor\g = ValF(StringField(mtlStr, 3, " "))
                    *tmpColor\b = ValF(StringField(mtlStr, 4, " "))                   
                    *tmpColor\a = 0.0  
                EndSelect 
                
              Wend
            EndIf
            
          Case "f"  
            Protected vts1.s = LCase(StringField(DataStr, 2, " "))
            Protected vts2.s = LCase(StringField(DataStr, 3, " "))
            Protected vts3.s = LCase(StringField(DataStr, 4, " "))            
            
            AddElement(tmpIndexList())
            tmpIndexList() = vts3
            AddElement(tmpIndexList())
            tmpIndexList() = vts2
            AddElement(tmpIndexList())            
            tmpIndexList() = vts1            
      EndSelect      
    Wend
  EndIf
  
  
  ResetList(tmpVertex())
  While NextElement(tmpVertex())
    *this\Primitive\AddVertex(tmpVertex()\x, tmpVertex()\y, tmpVertex()\z * -1.0)
  Wend
   

  ResetList(tmpIndexList())
  While NextElement(tmpIndexList())
    Index.l       = Val(StringField(tmpIndexList(), 1, "/")) - 1 
    UVIndex.l     = Val(StringField(tmpIndexList(), 2, "/")) - 1
    NormalIndex.l = Val(StringField(tmpIndexList(), 3, "/")) - 1
    
    *this\Primitive\AddIndex(Index)
    *this\Primitive\SetColor(*tmpColor) 
    *this\Primitive\SelectVertex(Index)
    
    If ListSize(tmpUVMap())
      SelectElement(tmpUVMap(), UVIndex)
      *this\Primitive\SetUV(tmpUVMap()\u,  1.0 - tmpUVMap()\v)
    EndIf
    SelectElement(tmpNormals(), NormalIndex)
    *this\Primitive\SetNormal(tmpNormals()\x, tmpNormals()\y, tmpNormals()\z * -1.0)

  Wend  
  
  
  ProcedureReturn #True
EndProcedure





; Create the Model an releases the primitive object
Procedure Model_SetModel(*this.Model_Template)
  *this\VertexArray = *this\Primitive\GetVerticesArray() 
  *this\IntexArray  = *this\Primitive\GetIndexArray()
  
  *this\Mesh\SetStride(*this\Primitive\VertexSize())
  *this\Primitive\Release()
  *this\Primitive = #Null    
  *this\Mesh\CreateBuffer(*this\VertexArray, *this\IntexArray)  
EndProcedure



; Render the model
Procedure Model_Render(*this.Model_Template)  
  *this\Mesh\Render()    
EndProcedure

Procedure Model_Release(*this.Model_Template) 
  *this\Mesh\Release()
  If *this\Primitive    : *this\Primitive\Release()     : EndIf  
  If *this\VertexArray  : FreeMemory(*this\VertexArray) : EndIf
  If *this\IntexArray   : FreeMemory(*this\IntexArray)  : EndIf  
  If *this\Texture      : *this\Texture\Release()       : EndIf
  FreeMemory(*this)
EndProcedure






;-//    Private procedures    //

;/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;-//    DataSection    //
DataSection
  Model_Class:   
  Data.i @Model_GetPrimitive()
  Data.i @Model_GetMesh()
  Data.i @Model_LoadTexture()
  Data.i @Model_AddTriangle()
  Data.i @Model_AddRectangle()
  Data.i @Model_AddCube()
  Data.i @Model_LoadObjModel()
  Data.i @Model_SetModel()
  Data.i @Model_Render()
  Data.i @Model_Release()
EndDataSection 

; IDE Options = PureBasic 5.20 beta 14 LTS (Windows - x64)
; CursorPosition = 468
; FirstLine = 427
; Folding = --
; EnableXP
; EnableCompileCount = 0
; EnableBuildCount = 0